<?php
/**
 * imp.php - performs an AJAX-requested action and returns the DIMP-specific
 * JSON object
 *
 * $Horde: dimp/imp.php,v 1.181 2007/07/18 01:41:38 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

function _generateDeleteResult($folder, $indices, $change)
{
    global $prefs;
    $result = new stdClass;
    $result->folder = $folder;
    $result->uids = DIMP::toRangeString($indices);
    $result->remove = ($prefs->getValue('use_vtrash') ||
                       $prefs->getValue('hide_deleted') ||
                       $prefs->getValue('use_trash'));
    $result->cacheid = _cacheID($folder);

    if ($change) {
        $result->rows = _getListMessages($folder, $indices, true);
    }

    return $result;
}

function _cacheID($folder)
{
    require_once IMP_BASE . '/lib/Mailbox.php';
    $imp_mailbox = &IMP_Mailbox::singleton($folder);
    return $imp_mailbox->getCacheId();
}

function _changed($folder, $compare, $indices = array())
{
    if (_cacheID($folder) != $compare) {
        return true;
    }

    if (!empty($indices)) {
        $sort = IMP::getSort($folder);
        if ($sort['by'] == SORTTHREAD) {
            require_once IMP_BASE . '/lib/Mailbox.php';
            $imp_mailbox = &IMP_Mailbox::singleton($folder);
            $threadob = $imp_mailbox->getThreadOb();
            foreach ($indices as $val) {
                if ($threadob->getThreadBase($val) !== false) {
                    return true;
                }
            }
        }
    }

    return false;
}

function _getListMessages($folder, $indices, $change)
{
    $args = array(
        'buffer_size' => intval(Util::getPost('buffer_size')),
        'cacheid' => Util::getPost('cacheid'),
        'filter' => Util::getPost('filter'),
        'folder' => $folder,
        'nearing' => Util::getPost('nearing'),
        'page_size' => intval(Util::getPost('page_size')),
        'searchfolder' => Util::getPost('searchfolder'),
        'searchmsg' => Util::getPost('searchmsg'),
    );

    if (!empty($indices)) {
        $args['uid'] = $indices[0];
    } else {
        $args['offset'] = intval(Util::getPost('offset'));
    }

    require_once DIMP_BASE . '/lib/Views/ListMessages.php';
    $list_msg = new DIMP_Views_ListMessages();
    $res = $list_msg->ListMessages($args);
    // TODO: This can potentially be optimized for arrival time sort - if the
    // cache ID changes, we know the changes must occur at end of mailbox.
    if (Util::getPost('update') || $change) {
        $res->update = true;
    }
    return $res;
}

if (empty($_SERVER['PATH_INFO'])) {
    // This is the only case where we really don't return anything, since the
    // frontend can be presumed not to make this request on purpose. Other
    // missing data cases we return a response of boolean false.
    exit;
}

// The following actions do not need write access to the session and should be
// opened read-only for performance reasons.
$read_only = array(
    'chunkContent', 'ShowPreview', 'Html2Text', 'Text2Html', 'GetReplyData', 'ShowOptions'
);

$action = basename($_SERVER['PATH_INFO']);
if (in_array($action, $read_only)) {
    $session_control = 'readonly';
}
$session_timeout = 'json';

if ($action != 'ShowOptions') {
    $authentication = 'none';
    $load_imp = true;
    @define('AUTH_HANDLER', true);
}
@define('DIMP_BASE', dirname(__FILE__));
require_once DIMP_BASE . '/lib/base.php';

// Process common request variables.
$folder = Util::getPost('folder');
$indices = DIMP::parseRangeString(Util::getPost('uid'));
$indices_mbox = false;
if ($folder && !empty($indices)) {
    $indices_mbox = array($folder => $indices);
}
$cacheid = Util::getPost('cacheid');

// Open an output buffer to ensure that we catch errors that might break JSON
// encoding.
ob_start();

$result = false;

switch ($action) {
case 'CreateFolder':
case 'CreateSubfolder':
    if (empty($folder)) {
        break;
    }

    $parent = Util::getPost('parent');
    if (($action == 'CreateSubfolder') && !$parent) {
        break;
    }

    require_once IMP_BASE . '/lib/IMAP/Tree.php';
    $imptree = &IMP_Tree::singleton();
    $imptree->eltDiffStart();

    require_once IMP_BASE . '/lib/Folder.php';
    $imp_folder = &IMP_Folder::singleton();

    $new = String::convertCharset($folder, NLS::getCharset(), 'UTF7-IMAP');
    $new = $imptree->createMailboxName($parent, $new);
    if (is_a($new, 'PEAR_Error')) {
        $notification->push($new, 'horde.error');
        $result = false;
    } else {
        $result = $imp_folder->create($new, $prefs->getValue('subscribe'));
        if ($result) {
            $result = DIMP::getFolderResponse($imptree);
        }
    }
    break;

case 'DeleteFolder':
    if (empty($folder)) {
        break;
    }

    require_once IMP_BASE . '/lib/IMAP/Tree.php';
    $imptree = &IMP_Tree::singleton();
    $imptree->eltDiffStart();

    require_once IMP_BASE . '/lib/Folder.php';
    $imp_folder = &IMP_Folder::singleton();
    $result = $imp_folder->delete(array($folder));
    if ($result) {
        $result = DIMP::getFolderResponse($imptree);
    }
    break;

case 'RenameFolder':
    $old = Util::getPost('old_name');
    $new_parent = Util::getPost('new_parent');
    $new = Util::getPost('new_name');
    if (!$old || !$new) {
        break;
    }

    require_once IMP_BASE . '/lib/IMAP/Tree.php';
    $imptree = &IMP_Tree::singleton();
    $imptree->eltDiffStart();

    require_once IMP_BASE . '/lib/Folder.php';
    $imp_folder = &IMP_Folder::singleton();

    if (!empty($new_parent)) {
        $ns = IMP::getNamespace($new_parent);
        $new = $new_parent . $ns['delimiter'] . $new;
    } else {
        $ns = IMP::getNamespace($old);
        $lastDelimiter = strrpos($old, $ns['delimiter']);
        if ($lastDelimiter !== false) {
            $new = substr($old, 0, $lastDelimiter + 1) . $new;
        }
    }

    require_once 'Horde/String.php';
    $new = String::convertCharset($new, NLS::getCharset(), 'UTF7-IMAP');
    if ($old == $new) {
        break;
    }
    $result = $imp_folder->rename($old, $new);
    if ($result) {
        $result = DIMP::getFolderResponse($imptree);
    }
    break;

case 'EmptyFolder':
    if (empty($folder)) {
        break;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $imp_message->emptyMailbox(array($folder));
    $result = new stdClass;
    $result->mbox = $folder;
    break;

case 'MarkFolderSeen':
case 'MarkFolderUnseen':
    if (empty($folder)) {
        break;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $result = $imp_message->flagAllInMailbox(array('seen'),
                                             array($folder),
                                             $action == 'MarkFolderSeen');
    if ($result) {
        $result = new stdClass;
        $result->mbox = $folder;

        require_once IMP_BASE . '/lib/IMAP/Tree.php';
        $imptree = &IMP_Tree::singleton();
        $elt = $imptree->get($folder);
        if ($imptree->isPolled($elt)) {
            $info = $imptree->getElementInfo($folder);
            $result->u = isset($info['unseen']) ? $info['unseen'] : 0;
            $result->n = isset($info['newmsg']) ? $info['newmsg'] : 0;
        }
    }
    break;

case 'ListFolders':
    require_once IMP_BASE . '/lib/IMAP/Tree.php';

    /* Initialize the IMP_Tree object. */
    $imptree = &IMP_Tree::singleton();
    $mask = IMPTREE_FLIST_CONTAINER;
    if (!$prefs->getValue('subscribe')) {
        $mask |= IMPTREE_FLIST_UNSUB;
    }

    $result = DIMP::getFolderResponse($imptree, array('a' => $imptree->folderList($mask), 'c' => array(), 'd' => array()));
    break;

case 'CheckCache':
case 'PollFolders':
    $result = new stdClass;

    if ($action == 'PollFolders') {
        require_once IMP_BASE . '/lib/Fetchmail.php';
        $fm_account = new IMP_Fetchmail_Account();
        $fm_count = $fm_account->count();
        if (!empty($fm_count)) {
            IMP_Fetchmail::fetchMail(range(0, $fm_count - 1));
        }

        require_once IMP_BASE . '/lib/IMAP/Tree.php';
        $imptree = &IMP_Tree::singleton();

        $result->poll = array();
        foreach ($imptree->getPollList(true) as $val) {
            if ($info = $imptree->getElementInfo($val)) {
                $result->poll[$val] = array('u' => $info['unseen'], 'n' => $info['newmsg']);
            }
        }
    }

    if (!empty($folder) && _changed($folder, $cacheid)) {
        $result->rows = _getListMessages($folder, $indices, true);
    }
    break;

case 'ListMessages':
case 'SearchMessages':
    if (empty($folder)) {
        break;
    }

    /* Change sort preferences if necessary. */
    if ($action == 'ListMessages') {
        $sortby = Util::getPost('sortby');
        $sortdir = Util::getPost('sortdir');
        if (!is_null($sortby) || !is_null($sortdir)) {
            IMP::setSort($sortby, $sortdir, $folder);
        }
    }

    $result = new stdClass;
    $result->rows = _getListMessages($folder, $indices, _changed($folder, $cacheid));
    break;

case 'MoveMessage':
case 'CopyMessage':
    $to = Util::getPost('tofld');
    if (!$to || !$indices_mbox) {
        break;
    }

    if ($action == 'MoveMessage') {
        $change = _changed($folder, $cacheid, $indices);
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $result = $imp_message->copy($to,
                                 $action == 'MoveMessage'
                                     ? IMP_MESSAGE_MOVE
                                     : IMP_MESSAGE_COPY,
                                 $indices_mbox);
    if ($result && ($action == 'MoveMessage')) {
        $result = _generateDeleteResult($folder, $indices, $change);
    }
    break;

case 'MarkMessage':
    $flag = Util::getPost('messageFlag');
    if (!$flag || !$indices_mbox) {
        break;
    }
    if ($flag[0] == '-') {
        $flag = substr($flag, 1);
        $set = false;
    } else {
        $set = true;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    $result = $imp_message->flag(array($flag), $indices_mbox, $set);
    if ($result) {
        $result = new stdClass;
    }
    break;

case 'DeleteMessage':
case 'UndeleteMessage':
    if (empty($indices_mbox)) {
        break;
    }

    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = new IMP_Message();
    if ($action == 'DeleteMessage') {
        $change = _changed($folder, $cacheid, $indices);
        $result = $imp_message->delete($indices_mbox);
        if ($result) {
            $result = _generateDeleteResult($folder, $indices, $change);
        }
    } else {
        $result = $imp_message->undelete($indices_mbox);
        if ($result) {
            $result = new stdClass;
        }
    }
    break;

case 'AddContact':
    $email = Util::getPost('email');
    $name = Util::getPost('name');
    // Allow $name to be empty.
    if (empty($email)) {
        break;
    }

    $result = IMP::addAddress($email, $name);
    if (is_a($result, 'PEAR_Error')) {
        $notification->push($result, 'horde.error');
        $result = false;
    } else {
        $result = true;
        $notification->push(sprintf(_("%s was successfully added to your address book."), $name ? $name : $email), 'horde.success');
    }
    break;

case 'ReportSpam':
case 'ReportHam':
    $change = _changed($folder, $cacheid, $indices);
    require_once IMP_BASE . '/lib/Spam.php';
    $spam = new IMP_Spam();
    $result = $spam->reportSpam($indices_mbox,
                                $action == 'ReportSpam' ? 'spam' : 'notspam');
    if ($result) {
        $result = _generateDeleteResult($folder, $indices, $change);
    }
    break;

case 'Blacklist':
case 'Whitelist':
    if (empty($indices_mbox)) {
        break;
    }

    require_once IMP_BASE . '/lib/Filter.php';
    $imp_filter = new IMP_Filter();
    if ($action == 'Whitelist') {
        $imp_filter->whitelistMessage($indices_mbox, false);
    } else {
        $change = _changed($folder, $cacheid, $indices);
        if ($imp_filter->blacklistMessage($indices_mbox, false)) {
            $result = _generateDeleteResult($folder, $indices, $change);
        }
    }
    break;

case 'ShowPreview':
    if (count($indices) != 1 || !$folder) {
        break;
    }

    $args = array(
        'folder' => $folder,
        'fullmessage' => 0,
        'index' => $indices[0],
        'options' => array('new_window', 'print', 'msg_source'),
    );

    require_once DIMP_BASE . '/lib/Views/ShowMessage.php';
    $show_msg = new DIMP_Views_ShowMessage();
    $result = $show_msg->ShowMessage($args);
    break;

case 'Html2Text':
    require_once 'Horde/Text/Filter.php';
    $result = new stdClass;
    // Need to replace line endings or else IE won't display line endings
    // properly.
    $result->text = str_replace("\n", "\r\n", Text_Filter::filter(Util::getPost('text'), 'html2text'));
    break;

case 'Text2Html':
    require_once 'Horde/Text/Filter.php';
    $result = new stdClass;
    $result->text = Text_Filter::filter(Util::getPost('text'), 'text2html', array('parselevel' => TEXT_HTML_MICRO_LINKURL, 'class' => null, 'callback' => null));
    break;

case 'GetForwardData':
    require_once IMP_BASE . '/lib/Compose.php';
    require_once IMP_BASE . '/lib/MIME/Contents.php';
    require_once IMP_BASE . '/lib/UI/Compose.php';
    require_once 'Horde/MIME/Message.php';
    $header = array();
    $msg = $header = null;
    $idx_string = $indices[0] . IMP_IDX_SEP . $folder;

    $imp_compose = &IMP_Compose::singleton(Util::getPost('imp_compose'));
    $imp_contents = &IMP_Contents::singleton($idx_string);
    $imp_ui = new IMP_UI_Compose();
    $fwd_msg = $imp_ui->getForwardData($imp_compose, $imp_contents, Util::getPost('type'), $idx_string);
    $header = $fwd_msg['headers'];
    $header['replytype'] = 'forward';

    $result = new stdClass;
    // Can't open read-only since we need to store the message cache id.
    $result->imp_compose = $imp_compose->getMessageCacheId();
    $result->fwd_list = DIMP::getAttachmentInfo($imp_compose);
    $result->body = $fwd_msg['body'];
    $result->header = $header;
    $result->format = $fwd_msg['format'];
    $result->identity = $fwd_msg['identity'];
    break;

case 'GetReplyData':
    require_once IMP_BASE . '/lib/Compose.php';
    require_once IMP_BASE . '/lib/MIME/Contents.php';
    $idx_string = $indices[0] . IMP_IDX_SEP . $folder;
    $imp_compose = &IMP_Compose::singleton(Util::getPost('imp_compose'));
    $imp_contents = &IMP_Contents::singleton($idx_string);
    $reply_msg = $imp_compose->replyMessage('*', $imp_contents);
    $header = (Util::getPost('type') == 'reply') ? $reply_msg['headers']['reply'] : $reply_msg['headers']['reply_all'];
    $header['replytype'] = 'reply';

    $result = new stdClass;
    $result->format = $reply_msg['format'];
    $result->body = $reply_msg['body'];
    $result->header = $header;
    $result->identity = $reply_msg['identities'];
    break;

case 'DeleteDraft':
    $index = Util::getPost('index');
    if (empty($indices)) {
        break;
    }
    require_once IMP_BASE . '/lib/Message.php';
    $imp_message = &IMP_Message::singleton();
    $idx_array = array($index . IMP_IDX_SEP . IMP::folderPref($prefs->getValue('drafts_folder'), true));
    $imp_message->delete($idx_array, true);
    break;

case 'DeleteAttach':
    $atc = Util::getPost('atc_indices');
    if (!is_null($atc)) {
        require_once IMP_BASE . '/lib/Compose.php';
        $imp_compose = &IMP_Compose::singleton(Util::getPost('imp_compose'));
        $imp_compose->deleteAttachment($atc);
    }
    break;

case 'GetMsgUIDSlice':
    require_once IMP_BASE . '/lib/Mailbox.php';
    $imp_mailbox = &IMP_Mailbox::singleton($folder);
    $ids = $imp_mailbox->getSortedList();
    $result = new stdClass;
    $result->uids = DIMP::toRangeString(array_slice($ids['s'], intval(Util::getPost('start')) - 1, intval(Util::getPost('length'))));
    break;

case 'ShowPortal':
    // Load the block list. Blocks are located in $dimp_block_list.
    // KEY: Block label  VALUE: Horde_Block object
    require DIMP_BASE . '/config/portal.php';

    $blocks = array();
    foreach ($dimp_block_list as $title => $block) {
        if (is_a($block['ob'], 'Horde_Block')) {
            $content = $block['ob']->getContent();
            if (!empty($content)) {
                $entry = array(
                    'app' => $block['ob']->_app,
                    'content' => $content,
                    'title' => $title,
                );
                foreach (array('domid', 'class') as $val) {
                    if (!empty($block[$val])) {
                        $entry[$val] = $block[$val];
                    }
                }
                if (!empty($block['tag'])) {
                    $entry[$block['tag']] = true;
                }
                $blocks[] = $entry;
            }
        }
    }

    $result = new stdClass;
    $result->portal = '';
    if (!empty($blocks)) {
        $t = new IMP_Template(DIMP_TEMPLATES . '/imp/');
        $t->set('block', $blocks);
        $result->portal = $t->fetch('portal.html');
    }
    break;

case 'ShowOptions':
    // Run preference code and load UI.
    ob_start();
    $chunk = true;
    include HORDE_BASE . '/services/prefs.php';

    // Generate output.
    $result = new stdClass;
    $result->portal .= ob_get_clean() . '<script type="text/javascript">' . file_get_contents(DIMP_BASE . '/js/prefs.js') . '</script>';
    break;

case 'chunkContent':
    $chunk = basename(Util::getPost('chunk'));
    if (!empty($chunk)) {
        $result = new stdClass;
        $result->chunk = Util::bufferOutput('include', DIMP_TEMPLATES . '/chunks/' . $chunk . '.php');
    }
    break;
}

// Clear the output buffer that we started above, and log any unexpected
// output at a DEBUG level.
$errors = ob_get_clean();
if ($errors) {
    Horde::logMessage('DIMP: unexpected output: ' .
                      $errors, __FILE__, __LINE__, PEAR_LOG_DEBUG);
}

// Send the final result.
IMP::sendHTTPResponse(DIMP::getNotificationResponse($result), 'json');
